// src/view/Mafs.tsx
import * as React5 from "react";

// src/context/CoordinateContext.tsx
import * as React from "react";
import invariant from "tiny-invariant";
var CoordinateContext = React.createContext(null);
CoordinateContext.displayName = "CoordinateContext";
function useCoordinateContext() {
  const context = React.useContext(CoordinateContext);
  invariant(
    context,
    "CoordinateContext is not loaded. Are you rendering a Mafs component outside of Mafs?"
  );
  return context;
}
var CoordinateContext_default = CoordinateContext;

// src/context/PaneContext.tsx
import * as React2 from "react";

// src/math.ts
function round(value, precision = 0) {
  const multiplier = Math.pow(10, precision || 0);
  return Math.round(value * multiplier) / multiplier;
}
function range(min, max, step = 1) {
  const result = [];
  for (let i = min; i < max - step / 2; i += step) {
    result.push(i);
  }
  const computedMax = result[result.length - 1] + step;
  if (Math.abs(max - computedMax) < step / 1e-6) {
    result.push(max);
  } else {
    result.push(computedMax);
  }
  return result;
}
function clamp(number, min, max) {
  return Math.min(Math.max(number, min), max);
}

// src/context/PaneContext.tsx
import { jsx } from "react/jsx-runtime";
var { round: round2, ceil, floor, log2 } = Math;
var PaneContext = React2.createContext({
  xPanes: [],
  yPanes: [],
  xPaneRange: [0, 0],
  yPaneRange: [0, 0]
});
PaneContext.displayName = "PaneContext";
function usePaneContext() {
  return React2.useContext(PaneContext);
}
function PaneManager({ children }) {
  const { xMin, xMax, yMin, yMax } = useCoordinateContext();
  const xPaneSize = 2 ** round2(log2(xMax - xMin) - 1);
  const yPaneSize = 2 ** round2(log2(yMax - yMin) - 1);
  const pad = 1 / 8;
  const xLowerBound = xPaneSize * floor(xMin / xPaneSize - pad);
  const xUpperBound = xPaneSize * ceil(xMax / xPaneSize + pad);
  const yLowerBound = yPaneSize * floor(yMin / yPaneSize - pad);
  const yUpperBound = yPaneSize * ceil(yMax / yPaneSize + pad);
  const xPanes = React2.useMemo(
    () => range(xLowerBound, xUpperBound - xPaneSize, xPaneSize).map(
      (xMin2) => [xMin2, xMin2 + xPaneSize]
    ),
    [xLowerBound, xUpperBound, xPaneSize]
  );
  const yPanes = React2.useMemo(
    () => range(yLowerBound, yUpperBound - yPaneSize, yPaneSize).map(
      (yMin2) => [yMin2, yMin2 + yPaneSize]
    ),
    [yLowerBound, yUpperBound, yPaneSize]
  );
  const context = React2.useMemo(
    () => ({
      xPanes,
      yPanes,
      xPaneRange: [xLowerBound, xUpperBound],
      yPaneRange: [yLowerBound, yUpperBound]
    }),
    [xPanes, yPanes, xLowerBound, xUpperBound, yLowerBound, yUpperBound]
  );
  return /* @__PURE__ */ jsx(PaneContext.Provider, { value: context, children });
}
PaneManager.displayName = "PaneManager";
var PaneContext_default = PaneManager;

// src/view/Mafs.tsx
import useResizeObserver from "use-resize-observer";
import { useDrag } from "@use-gesture/react";

// src/vec.ts
var vec;
((vec2) => {
  function add(v, v2) {
    return [v[0] + v2[0], v[1] + v2[1]];
  }
  vec2.add = add;
  function sub(v, v2) {
    return [v[0] - v2[0], v[1] - v2[1]];
  }
  vec2.sub = sub;
  function mag(v) {
    return Math.sqrt(v[0] * v[0] + v[1] * v[1]);
  }
  vec2.mag = mag;
  function normal(v) {
    return [-v[1], v[0]];
  }
  vec2.normal = normal;
  function lerp(v1, v2, t) {
    const d = sub(v2, v1);
    const m = mag(d);
    return add(v1, withMag(d, t * m));
  }
  vec2.lerp = lerp;
  function withMag(v, m) {
    const magnitude = mag(v);
    return scale(v, m / magnitude);
  }
  vec2.withMag = withMag;
  function normalize(v) {
    return withMag(v, 1);
  }
  vec2.normalize = normalize;
  function scale(v, sc) {
    return [v[0] * sc, v[1] * sc];
  }
  vec2.scale = scale;
  function transform(v, m) {
    return [v[0] * m[0] + v[1] * m[1] + m[2], v[0] * m[3] + v[1] * m[4] + m[5]];
  }
  vec2.transform = transform;
  function matrixMult(m, m2) {
    return matrixCreate(
      m[0] * m2[0] + m[1] * m2[3],
      m[3] * m2[0] + m[4] * m2[3],
      m[0] * m2[1] + m[1] * m2[4],
      m[3] * m2[1] + m[4] * m2[4],
      m[0] * m2[2] + m[1] * m2[5] + m[2],
      m[3] * m2[2] + m[4] * m2[5] + m[5]
    );
  }
  vec2.matrixMult = matrixMult;
  function rotate(v, a) {
    const c = Math.cos(a);
    const s = Math.sin(a);
    return [v[0] * c - v[1] * s, v[0] * s + v[1] * c];
  }
  vec2.rotate = rotate;
  function rotateAbout(v, cp, a) {
    const v2 = sub(v, cp);
    return add(cp, rotate(v2, a));
  }
  vec2.rotateAbout = rotateAbout;
  function midpoint(v, v2) {
    return lerp(v, v2, 0.5);
  }
  vec2.midpoint = midpoint;
  function dist(v, v2) {
    return Math.sqrt(squareDist(v, v2));
  }
  vec2.dist = dist;
  function squareDist(v, v2) {
    return Math.pow(v2[0] - v[0], 2) + Math.pow(v2[1] - v[1], 2);
  }
  vec2.squareDist = squareDist;
  function dot(v, v2) {
    return v[0] * v2[0] + v[1] * v2[1];
  }
  vec2.dot = dot;
  function det(m) {
    return m[0] * m[4] - m[3] * m[1];
  }
  vec2.det = det;
  function matrixInvert(a) {
    const mDet = det(a);
    if (!mDet)
      return null;
    const invDet = 1 / mDet;
    const a00 = a[0], a01 = a[1], a02 = a[2];
    const a10 = a[3], a11 = a[4], a12 = a[5];
    return matrixCreate(
      invDet * a11,
      invDet * -a10,
      invDet * -a01,
      invDet * a00,
      invDet * (a12 * a01 - a02 * a11),
      invDet * (-a12 * a00 + a02 * a10)
    );
  }
  vec2.matrixInvert = matrixInvert;
  function matrixBuilder(m = null) {
    const _m = m || matrixCreate();
    return {
      mult: (m2) => matrixBuilder(matrixMult(m2, _m)),
      translate: (x, y) => matrixBuilder(matrixMult(matrixCreate(1, 0, 0, 1, x, y), _m)),
      rotate: (a) => {
        const c = Math.cos(a);
        const s = Math.sin(a);
        return matrixBuilder(matrixMult(matrixCreate(c, s, -s, c), _m));
      },
      scale: (x, y) => matrixBuilder(matrixMult(matrixCreate(x, 0, 0, y), _m)),
      shear: (x, y) => matrixBuilder(matrixMult(matrixCreate(1, y, x, 1), _m)),
      get: () => [..._m]
    };
  }
  vec2.matrixBuilder = matrixBuilder;
  function toCSS(matrix) {
    const [a, c, tx, b, d, ty] = matrix;
    return `matrix(${a}, ${b}, ${c}, ${d}, ${tx}, ${ty})`;
  }
  vec2.toCSS = toCSS;
  vec2.identity = matrixBuilder().get();
})(vec || (vec = {}));
function matrixCreate(a = 1, b = 0, c = 0, d = 1, tx = 0, ty = 0) {
  return [a, c, tx, b, d, ty];
}

// src/context/TransformContext.tsx
import * as React3 from "react";
import invariant2 from "tiny-invariant";
var TransformContext = React3.createContext(null);
TransformContext.displayName = "TransformContext";
function useTransformContext() {
  const context = React3.useContext(TransformContext);
  invariant2(
    context,
    "TransformContext is not loaded. Are you rendering a Mafs component outside of a MafsView?"
  );
  return context;
}

// src/context/SpanContext.tsx
import * as React4 from "react";
import invariant3 from "tiny-invariant";
var SpanContext = React4.createContext({
  xSpan: 0,
  ySpan: 0
});
SpanContext.displayName = "SpanContext";
function useSpanContext() {
  const context = React4.useContext(SpanContext);
  invariant3(context, "SpanContext is not defined");
  return context;
}

// src/view/Mafs.tsx
import { jsx as jsx2 } from "react/jsx-runtime";
function Mafs({
  width: desiredWidth = "auto",
  height = 500,
  pan = true,
  viewBox = { x: [-3, 3], y: [-3, 3] },
  preserveAspectRatio = "contain",
  children,
  ssr = false
}) {
  const [visible, setVisible] = React5.useState(ssr ? true : false);
  const desiredCssWidth = desiredWidth === "auto" ? "100%" : `${desiredWidth}px`;
  const { ref, width = ssr ? 500 : 1 } = useResizeObserver();
  React5.useEffect(() => {
    setVisible(true);
  }, []);
  const aspect = width / height;
  const [offset, setOffset] = React5.useState([0, 0]);
  const padding = viewBox?.padding ?? 0.5;
  const aoi = {
    xMin: (viewBox?.x?.[0] ?? 0) - padding + offset[0],
    xMax: (viewBox?.x?.[1] ?? 0) + padding + offset[0],
    yMin: (viewBox?.y?.[0] ?? 0) - padding + offset[1],
    yMax: (viewBox?.y?.[1] ?? 0) + padding + offset[1]
  };
  let xMin = aoi.xMin;
  let xMax = aoi.xMax;
  let yMin = aoi.yMin;
  let yMax = aoi.yMax;
  if (preserveAspectRatio === "contain") {
    const aoiAspect = (aoi.xMax - aoi.xMin) / (aoi.yMax - aoi.yMin);
    if (aoiAspect > aspect) {
      const yCenter = (aoi.yMax + aoi.yMin) / 2;
      const ySpan2 = (aoi.xMax - aoi.xMin) / aspect / 2;
      yMin = yCenter - ySpan2;
      yMax = yCenter + ySpan2;
    } else {
      const xCenter = (aoi.xMax + aoi.xMin) / 2;
      const xSpan2 = (aoi.yMax - aoi.yMin) * aspect / 2;
      xMin = xCenter - xSpan2;
      xMax = xCenter + xSpan2;
    }
  }
  const xSpan = xMax - xMin;
  const ySpan = yMax - yMin;
  const bind = useDrag(
    ({ offset: [mx, my], event, type }) => {
      if (type.includes("key"))
        event.preventDefault();
      setOffset([-mx / width * xSpan, my / height * ySpan]);
    },
    { enabled: pan }
  );
  const viewTransform = React5.useMemo(() => {
    const scaleX = round(1 / xSpan * width, 5);
    const scaleY = round(-1 / ySpan * height, 5);
    return vec.matrixBuilder().scale(scaleX, scaleY).get();
  }, [height, width, xSpan, ySpan]);
  const toPxCSS = vec.toCSS(viewTransform);
  const coordinateContext = React5.useMemo(
    () => ({ xMin, xMax, yMin, yMax, height, width }),
    [xMin, xMax, yMin, yMax, height, width]
  );
  const viewBoxX = round(xMin / (xMax - xMin) * width, 2);
  const viewBoxY = round(yMax / (yMin - yMax) * height, 2);
  return /* @__PURE__ */ jsx2(
    "div",
    {
      className: "MafsView",
      style: { width: desiredCssWidth },
      tabIndex: pan ? 0 : -1,
      ref,
      ...bind(),
      children: /* @__PURE__ */ jsx2(CoordinateContext_default.Provider, { value: coordinateContext, children: /* @__PURE__ */ jsx2(SpanContext.Provider, { value: { xSpan, ySpan }, children: /* @__PURE__ */ jsx2(
        TransformContext.Provider,
        {
          value: { userTransform: vec.identity, viewTransform },
          children: /* @__PURE__ */ jsx2(PaneContext_default, { children: /* @__PURE__ */ jsx2(
            "svg",
            {
              width,
              height,
              viewBox: `${viewBoxX} ${viewBoxY} ${width} ${height}`,
              preserveAspectRatio: "xMidYMin",
              style: {
                width: desiredCssWidth,
                touchAction: pan ? "none" : "auto",
                ...{
                  "--mafs-view-transform": toPxCSS,
                  "--mafs-user-transform": "translate(0, 0)"
                }
              },
              children: visible && children
            }
          ) })
        }
      ) }) })
    }
  );
}
Mafs.displayName = "Mafs";

// src/display/Coordinates/Axes.tsx
import { Fragment, jsx as jsx3, jsxs } from "react/jsx-runtime";
var defaultLabelMaker = (x) => /* @__PURE__ */ jsxs(Fragment, { children: [
  x,
  x < 0 && /* @__PURE__ */ jsx3("tspan", { visibility: "hidden", children: "-" })
] });
var defaultAxisOptions = {
  axis: true,
  lines: 1,
  labels: defaultLabelMaker
};
function XLabels({ separation, labelMaker }) {
  const { viewTransform } = useTransformContext();
  const { xPanes } = usePaneContext();
  return /* @__PURE__ */ jsx3("g", { className: "mafs-axis", children: xPanes.map(([min, max]) => /* @__PURE__ */ jsx3("g", { children: snappedRange(min, max - separation, separation).filter((x) => Math.abs(x) > separation / 1e6).map((x) => /* @__PURE__ */ jsx3(
    "text",
    {
      x: vec.transform([x, 0], viewTransform)[0],
      y: 5,
      dominantBaseline: "hanging",
      textAnchor: "middle",
      children: labelMaker(x)
    },
    x
  )) }, `${min},${max}`)) });
}
XLabels.displayName = "CartesianCoordinates.XLabels";
function YLabels({ separation, labelMaker }) {
  const { viewTransform } = useTransformContext();
  const { yPanes } = usePaneContext();
  return /* @__PURE__ */ jsx3("g", { className: "mafs-axis", children: yPanes.map(([min, max]) => /* @__PURE__ */ jsx3("g", { children: snappedRange(min, max - separation, separation).filter((y) => Math.abs(y) > separation / 1e6).map((y) => /* @__PURE__ */ jsx3(
    "text",
    {
      x: 5,
      y: vec.transform([0, y], viewTransform)[1],
      dominantBaseline: "central",
      children: labelMaker(y)
    },
    y
  )) }, `${min},${max}`)) });
}
YLabels.displayName = "CartesianCoordinates.YLabels";

// src/display/Coordinates/Cartesian.tsx
import { Fragment as Fragment2, jsx as jsx4, jsxs as jsxs2 } from "react/jsx-runtime";
var incrementer = 0;
function Cartesian({
  xAxis: xAxisOverrides,
  yAxis: yAxisOverrides,
  subdivisions = false
}) {
  const xAxisEnabled = xAxisOverrides !== false;
  const yAxisEnabled = yAxisOverrides !== false;
  const xAxis = { subdivisions, ...defaultAxisOptions, ...xAxisOverrides };
  const yAxis = { subdivisions, ...defaultAxisOptions, ...yAxisOverrides };
  const id = `cartesian-${incrementer++}`;
  const { viewTransform } = useTransformContext();
  const { xPaneRange, yPaneRange } = usePaneContext();
  const [xMin, xMax] = xPaneRange;
  const [yMin, yMax] = yPaneRange;
  const [vxMin, vyMin] = vec.transform([xMin, yMin], viewTransform);
  const [vxMax, vyMax] = vec.transform([xMax, yMax], viewTransform);
  const xLines = xAxis.lines || 1;
  const yLines = yAxis.lines || 1;
  const [unitW, unitH] = vec.transform([xLines, -yLines], viewTransform);
  const xSubs = xAxis.subdivisions || 1;
  const ySubs = yAxis.subdivisions || 1;
  const subUnitW = unitW / xSubs;
  const subUnitH = unitH / ySubs;
  return /* @__PURE__ */ jsxs2("g", { fill: "none", children: [
    /* @__PURE__ */ jsxs2("pattern", { x: 0, y: 0, width: unitW, height: unitH, id, patternUnits: "userSpaceOnUse", children: [
      /* @__PURE__ */ jsx4(
        "pattern",
        {
          width: subUnitW,
          height: subUnitH,
          id: `${id}-subdivision`,
          patternUnits: "userSpaceOnUse",
          children: /* @__PURE__ */ jsxs2("g", { stroke: "var(--grid-line-subdivision-color)", children: [
            xAxisEnabled !== false && xSubs > 1 && /* @__PURE__ */ jsxs2(Fragment2, { children: [
              /* @__PURE__ */ jsx4("line", { x1: 0, y1: 0, x2: 0, y2: subUnitH }),
              /* @__PURE__ */ jsx4("line", { x1: subUnitW, y1: 0, x2: subUnitW, y2: subUnitH })
            ] }),
            yAxisEnabled !== false && ySubs > 1 && /* @__PURE__ */ jsxs2(Fragment2, { children: [
              /* @__PURE__ */ jsx4("line", { x1: 0, y1: 0, x2: subUnitW, y2: 0 }),
              /* @__PURE__ */ jsx4("line", { x1: 0, y1: subUnitH, x2: subUnitW, y2: subUnitH })
            ] })
          ] })
        }
      ),
      /* @__PURE__ */ jsx4("rect", { width: unitW, height: unitH, fill: `url(#${id}-subdivision)` }),
      /* @__PURE__ */ jsxs2("g", { stroke: "var(--mafs-line-color)", children: [
        yAxisEnabled && xAxis.lines && /* @__PURE__ */ jsxs2(Fragment2, { children: [
          /* @__PURE__ */ jsx4("line", { x1: 0, y1: 0, x2: unitW, y2: 0 }),
          /* @__PURE__ */ jsx4("line", { x1: 0, y1: unitH, x2: unitW, y2: unitH })
        ] }),
        xAxisEnabled && yAxis.lines && /* @__PURE__ */ jsxs2(Fragment2, { children: [
          /* @__PURE__ */ jsx4("line", { x1: 0, y1: 0, x2: 0, y2: unitH }),
          /* @__PURE__ */ jsx4("line", { x1: unitW, y1: 0, x2: unitW, y2: unitH })
        ] })
      ] })
    ] }),
    /* @__PURE__ */ jsx4("rect", { x: vxMin, y: vyMax, width: vxMax - vxMin, height: vyMin - vyMax, fill: `url(#${id})` }),
    /* @__PURE__ */ jsxs2("g", { stroke: "var(--mafs-origin-color)", children: [
      xAxisEnabled && xAxis.axis && /* @__PURE__ */ jsx4("line", { x1: vxMin, y1: 0, x2: vxMax, y2: 0 }),
      yAxisEnabled && yAxis.axis && /* @__PURE__ */ jsx4("line", { x1: 0, y1: vyMin, x2: 0, y2: vyMax })
    ] }),
    /* @__PURE__ */ jsxs2("g", { className: "mafs-shadow", children: [
      xAxisEnabled && xAxis.labels && /* @__PURE__ */ jsx4(XLabels, { separation: xAxis.lines || 1, labelMaker: xAxis.labels }),
      yAxisEnabled && yAxis.labels && /* @__PURE__ */ jsx4(YLabels, { separation: yAxis.lines || 1, labelMaker: yAxis.labels })
    ] })
  ] });
}
function snappedRange(min, max, step) {
  return range(Math.floor(min / step) * step, Math.ceil(max / step) * step, step);
}
function autoPi(x) {
  if (x === 0)
    return "0";
  if (Math.abs(Math.PI - x) < 1e-3)
    return "\u03C0";
  if (Math.abs(-Math.PI - x) < 1e-3)
    return "-\u03C0";
  return `${round(x / Math.PI, 5)}\u03C0`;
}

// src/display/Coordinates/Polar.tsx
import { jsx as jsx5, jsxs as jsxs3 } from "react/jsx-runtime";
var thetas = range(0, 2 * Math.PI, Math.PI / 12);
function PolarCoordinates({
  xAxis: xAxisOverrides,
  yAxis: yAxisOverrides,
  lines = 1,
  subdivisions
}) {
  const xAxisEnabled = xAxisOverrides !== false;
  const yAxisEnabled = yAxisOverrides !== false;
  const xAxis = { ...defaultAxisOptions, ...xAxisOverrides };
  const yAxis = { ...defaultAxisOptions, ...yAxisOverrides };
  const { viewTransform } = useTransformContext();
  const { xPaneRange, yPaneRange } = usePaneContext();
  const [xMin, xMax] = xPaneRange;
  const [yMin, yMax] = yPaneRange;
  const distances = [
    vec.mag([xMin, yMin]),
    vec.mag([xMin, yMax]),
    vec.mag([xMax, yMin]),
    vec.mag([xMax, yMax]),
    vec.mag([(xMin + xMax) / 2, (yMin + yMax) / 2])
  ];
  const b = lines;
  const closeToOrigin = Math.min(...distances) < Math.max(xMax - xMin, yMax - yMin);
  const minRadiusPrecise = closeToOrigin ? 0 : Math.min(...distances);
  const maxRadiusPrecise = Math.max(...distances);
  const minRadius = Math.floor(minRadiusPrecise / b) * b;
  const maxRadius = Math.ceil(maxRadiusPrecise / b) * b;
  const [vxMin, vyMin] = vec.transform([xMin, yMin], viewTransform);
  const [vxMax, vyMax] = vec.transform([xMax, yMax], viewTransform);
  const [scaleX, scaleY] = vec.transform([1, -1], viewTransform);
  const rs = range(minRadius, maxRadius, b);
  const subRs = subdivisions != void 0 ? range(minRadius, maxRadius, b / subdivisions) : [];
  return /* @__PURE__ */ jsxs3("g", { fill: "none", children: [
    /* @__PURE__ */ jsx5("g", { stroke: "var(--grid-line-subdivision-color)", children: thetas.map((theta) => /* @__PURE__ */ jsx5(
      "line",
      {
        x1: 0,
        y1: 0,
        x2: Math.cos(theta) * maxRadius * scaleX,
        y2: -Math.sin(theta) * maxRadius * scaleY
      },
      theta
    )) }),
    subRs.map((r) => /* @__PURE__ */ jsx5(
      "ellipse",
      {
        "data-r": r,
        cx: 0,
        cy: 0,
        rx: r * scaleX,
        ry: r * scaleY,
        stroke: "var(--grid-line-subdivision-color)"
      },
      r
    )),
    rs.map((r) => /* @__PURE__ */ jsx5(
      "ellipse",
      {
        "data-r": r,
        cx: 0,
        cy: 0,
        rx: r * scaleX,
        ry: r * scaleY,
        stroke: "var(--mafs-line-color)"
      },
      r
    )),
    /* @__PURE__ */ jsxs3("g", { stroke: "var(--mafs-origin-color)", children: [
      xAxisEnabled && xAxis.axis && /* @__PURE__ */ jsx5("line", { x1: vxMin, y1: 0, x2: vxMax, y2: 0 }),
      yAxisEnabled && yAxis.axis && /* @__PURE__ */ jsx5("line", { x1: 0, y1: vyMin, x2: 0, y2: vyMax })
    ] }),
    /* @__PURE__ */ jsxs3("g", { className: "mafs-shadow", children: [
      xAxisEnabled && xAxis.labels && /* @__PURE__ */ jsx5(
        XLabels,
        {
          separation: xAxisOverrides?.lines || lines || 1,
          labelMaker: xAxis.labels || defaultLabelMaker
        }
      ),
      yAxisEnabled && yAxis.labels && /* @__PURE__ */ jsx5(
        YLabels,
        {
          separation: yAxisOverrides?.lines || lines || 1,
          labelMaker: yAxis.labels || defaultLabelMaker
        }
      )
    ] })
  ] });
}

// src/display/Coordinates/index.tsx
PolarCoordinates.displayName = "Coordinates.Polar";
Cartesian.displayName = "Coordinates.Cartesian";
var Coordinates = {
  Cartesian,
  Polar: PolarCoordinates
};

// src/display/Plot/Simple.tsx
import * as React7 from "react";

// src/display/Plot/Parametric.tsx
import * as React6 from "react";

// src/display/Plot/PlotUtils.tsx
function adaptiveSampling(xy, t, minSamplingDepth, maxSamplingDepth, errorThreshold) {
  let result = "M ";
  function subdivide(min, max, pushLeft, pushRight, depth, xyMinX2, xyMinY2, xyMaxX2, xyMaxY2) {
    const t2 = 0.5;
    const mid = min + (max - min) * t2;
    const [xyMidX, xyMidY] = xy(mid);
    if (depth < minSamplingDepth) {
      subdivide(min, mid, true, false, depth + 1, xyMinX2, xyMinY2, xyMidX, xyMidY);
      subdivide(mid, max, false, true, depth + 1, xyMidX, xyMidY, xyMaxX2, xyMaxY2);
      return;
    }
    if (depth < maxSamplingDepth) {
      const xyLerpMid = vec.lerp([xyMinX2, xyMinY2], [xyMaxX2, xyMaxY2], t2);
      const error = vec.squareDist([xyMidX, xyMidY], xyLerpMid);
      if (error > errorThreshold) {
        subdivide(min, mid, true, false, depth + 1, xyMinX2, xyMinY2, xyMidX, xyMidY);
        subdivide(mid, max, false, true, depth + 1, xyMidX, xyMidY, xyMaxX2, xyMaxY2);
        return;
      }
    }
    if (pushLeft && Number.isFinite(xyMinX2) && Number.isFinite(xyMinY2)) {
      result += `${xyMinX2} ${xyMinY2} L `;
    }
    if (Number.isFinite(xyMidX) && Number.isFinite(xyMidY)) {
      result += `${xyMidX} ${xyMidY} L `;
    }
    if (pushRight && Number.isFinite(xyMaxX2) && Number.isFinite(xyMaxY2)) {
      result += `${xyMaxX2} ${xyMaxY2} L `;
    }
  }
  const [tMin, tMax] = t;
  const [xyMinX, xyMinY] = xy(tMin);
  const [xyMaxX, xyMaxY] = xy(tMax);
  subdivide(tMin, tMax, true, true, 0, xyMinX, xyMinY, xyMaxX, xyMaxY);
  return result.substring(0, result.length - 2);
}

// src/display/Plot/Parametric.tsx
import { jsx as jsx6 } from "react/jsx-runtime";
function Parametric({
  xy,
  t,
  color,
  style = "solid",
  weight = 2,
  opacity = 1,
  maxSamplingDepth = 14,
  minSamplingDepth = 8,
  svgPathProps = {}
}) {
  const { viewTransform } = useTransformContext();
  const pixelsPerSquare = -vec.det(viewTransform);
  const [tMin, tMax] = t;
  const errorThreshold = 0.1 / pixelsPerSquare;
  const svgPath = React6.useMemo(
    () => adaptiveSampling(xy, [tMin, tMax], minSamplingDepth, maxSamplingDepth, errorThreshold),
    [xy, minSamplingDepth, maxSamplingDepth, errorThreshold, tMin, tMax]
  );
  return /* @__PURE__ */ jsx6(
    "path",
    {
      d: svgPath,
      strokeWidth: weight,
      fill: "none",
      strokeLinecap: "round",
      strokeLinejoin: "round",
      strokeDasharray: style === "dashed" ? "1,10" : void 0,
      ...svgPathProps,
      style: {
        stroke: color || "var(--mafs-fg)",
        strokeOpacity: opacity,
        vectorEffect: "non-scaling-stroke",
        transform: "var(--mafs-view-transform)",
        ...svgPathProps.style || {}
      }
    }
  );
}

// src/display/Plot/Simple.tsx
import { jsx as jsx7 } from "react/jsx-runtime";
function OfX({ y, ...props }) {
  const {
    xPaneRange: [xMin, xMax]
  } = usePaneContext();
  const xy = React7.useCallback((x) => [x, y(x)], [y]);
  const t = React7.useMemo(() => [xMin, xMax], [xMin, xMax]);
  return /* @__PURE__ */ jsx7(Parametric, { xy, t, ...props });
}
function OfY({ x, ...props }) {
  const {
    yPaneRange: [yMin, yMax]
  } = usePaneContext();
  const xy = React7.useCallback((y) => [x(y), y], [x]);
  const t = React7.useMemo(() => [yMin, yMax], [yMin, yMax]);
  return /* @__PURE__ */ jsx7(Parametric, { xy, t, ...props });
}

// src/display/Theme.ts
var Theme = {
  foreground: "var(--mafs-fg)",
  background: "var(--mafs-bg)",
  red: "var(--mafs-red)",
  orange: "var(--mafs-orange)",
  green: "var(--mafs-green)",
  blue: "var(--mafs-blue)",
  indigo: "var(--mafs-indigo)",
  violet: "var(--mafs-violet)",
  pink: "var(--mafs-pink)",
  yellow: "var(--mafs-yellow)"
};

// src/display/Plot/VectorField.tsx
import { Fragment as Fragment3, jsx as jsx8 } from "react/jsx-runtime";
var xyOpacityDefault = () => 1;
function VectorField({
  xy,
  step = 1,
  xyOpacity = xyOpacityDefault,
  opacityStep = xyOpacity === xyOpacityDefault ? 1 : 0.2,
  color = Theme.foreground
}) {
  const { viewTransform: pixelMatrix } = useTransformContext();
  const { xPanes, yPanes } = usePaneContext();
  opacityStep = Math.min(1, Math.max(0.01, opacityStep));
  const opacityGrainularity = Math.ceil(1 / opacityStep);
  const layers = generateOpacityLayers(opacityGrainularity);
  function fieldForRegion(xMin, xMax, yMin, yMax) {
    for (let x = Math.floor(xMin); x <= Math.ceil(xMax); x += step) {
      for (let y = Math.floor(yMin); y <= Math.ceil(yMax); y += step) {
        const tail = [x, y];
        const trueOffset = xy([x, y]);
        const trueMag = vec.mag(trueOffset);
        const scaledOffset = vec.scale(vec.normalize(trueOffset), Math.min(trueMag, step * 0.75));
        const tip = vec.add(tail, scaledOffset);
        const pixelTail = vec.transform(tail, pixelMatrix);
        const pixelTipOffset = vec.transform(scaledOffset, pixelMatrix);
        const pixelSize = vec.mag(pixelTipOffset);
        const pixelTip = vec.transform(tip, pixelMatrix);
        const arrowVector = vec.scale(vec.normalize(pixelTipOffset), Math.min(pixelSize, 5));
        const left = vec.add(pixelTip, vec.rotate(arrowVector, 5 / 6 * Math.PI));
        const right = vec.add(pixelTip, vec.rotate(arrowVector, -(5 / 6) * Math.PI));
        const trueOpacity = xyOpacity([x, y]);
        const layer = findClosetLayer(layers, trueOpacity);
        layer.d += ` M ${pixelTail[0]} ${pixelTail[1]} L ${pixelTip[0]} ${pixelTip[1]}  L ${left[0]} ${left[1]}  L ${right[0]} ${right[1]}  L ${pixelTip[0]} ${pixelTip[1]} `;
      }
    }
  }
  for (const [xMin, xMax] of xPanes) {
    for (const [yMin, yMax] of yPanes) {
      fieldForRegion(xMin, xMax, yMin, yMax);
    }
  }
  return /* @__PURE__ */ jsx8(Fragment3, { children: layers.map((layer, index) => /* @__PURE__ */ jsx8(
    "path",
    {
      d: layer.d,
      style: {
        stroke: color,
        fill: color,
        opacity: layer.opacity,
        fillOpacity: layer.opacity,
        strokeOpacity: layer.opacity
      },
      strokeLinecap: "round",
      strokeLinejoin: "round"
    },
    index
  )) });
}
function generateOpacityLayers(opacityGrainularity) {
  const layers = [];
  const step = 1 / opacityGrainularity;
  for (let i = 1; i > 0; i -= step) {
    const layer = {
      d: "",
      opacity: i
    };
    layers.push(layer);
  }
  return layers;
}
function findClosetLayer(layers, pointOpacity) {
  pointOpacity = clamp(pointOpacity, 0, 1);
  const index = layers.length - 1 - Math.round(pointOpacity * (layers.length - 1));
  return layers[index];
}

// src/display/Plot.tsx
var Plot = {
  OfX,
  OfY,
  Parametric,
  VectorField
};
Plot.OfX.displayName = "Plot.OfX";
Plot.OfY.displayName = "Plot.OfY";
Plot.Parametric.displayName = "Plot.Parametric";
Plot.VectorField.displayName = "Plot.VectorField";

// src/display/Line/ThroughPoints.tsx
import { jsx as jsx9 } from "react/jsx-runtime";
function ThroughPoints({
  point1,
  point2,
  color = Theme.foreground,
  style = "solid",
  weight = 2,
  opacity = 1
}) {
  const { viewTransform, userTransform } = useTransformContext();
  const transform = vec.matrixMult(viewTransform, userTransform);
  const segment = vec.scale(vec.normalize(vec.sub(point2, point1)), 1e5);
  const scaledPoint1 = vec.transform(vec.sub(point1, segment), transform);
  const scaledPoint2 = vec.transform(vec.add(point2, segment), transform);
  return /* @__PURE__ */ jsx9(
    "line",
    {
      x1: round(scaledPoint1[0], 2),
      y1: round(scaledPoint1[1], 2),
      x2: round(scaledPoint2[0], 2),
      y2: round(scaledPoint2[1], 2),
      style: { stroke: color },
      strokeWidth: weight,
      opacity,
      strokeDasharray: style === "dashed" ? "4,3" : void 0
    }
  );
}

// src/display/Line/PointAngle.tsx
import { jsx as jsx10 } from "react/jsx-runtime";
function PointAngle({ point, angle, ...rest }) {
  const point2 = vec.add(point, vec.rotate([1, 0], angle));
  return /* @__PURE__ */ jsx10(ThroughPoints, { point1: point, point2, ...rest });
}

// src/display/Line/PointSlope.tsx
import { jsx as jsx11 } from "react/jsx-runtime";
function PointSlope({ point, slope, ...rest }) {
  return /* @__PURE__ */ jsx11(PointAngle, { point, angle: Math.atan(slope), ...rest });
}

// src/display/Line/Segment.tsx
import { jsx as jsx12 } from "react/jsx-runtime";
function Segment({
  point1,
  point2,
  color = Theme.foreground,
  style = "solid",
  weight = 2,
  opacity = 1
}) {
  const { viewTransform: pixelMatrix, userTransform } = useTransformContext();
  const transform = vec.matrixMult(pixelMatrix, userTransform);
  const scaledPoint1 = vec.transform(point1, transform);
  const scaledPoint2 = vec.transform(point2, transform);
  return /* @__PURE__ */ jsx12(
    "line",
    {
      x1: round(scaledPoint1[0], 2),
      y1: round(scaledPoint1[1], 2),
      x2: round(scaledPoint2[0], 2),
      y2: round(scaledPoint2[1], 2),
      style: { stroke: color },
      strokeWidth: weight,
      opacity,
      strokeDasharray: style === "dashed" ? "1,10" : void 0
    }
  );
}

// src/display/Line.tsx
var Line = {
  PointAngle,
  PointSlope,
  ThroughPoints,
  Segment
};
Line.PointAngle.displayName = "Line.PointAngle";
Line.PointSlope.displayName = "Line.PointSlope";
Line.Segment.displayName = "Line.Segment";
Line.ThroughPoints.displayName = "Line.ThroughPoints";

// src/display/Ellipse.tsx
import { jsx as jsx13 } from "react/jsx-runtime";
function Ellipse({
  center,
  radius,
  angle = 0,
  strokeStyle = "solid",
  strokeOpacity = 1,
  weight = 2,
  color = Theme.foreground,
  fillOpacity = 0.15,
  svgEllipseProps = {}
}) {
  const { viewTransform: toPx, userTransform } = useTransformContext();
  const transform = vec.matrixBuilder().translate(...center).mult(userTransform).scale(1, -1).mult(toPx).scale(1, -1).get();
  const cssTransform = `
    ${vec.toCSS(transform)}
    rotate(${angle * (180 / Math.PI)})
  `;
  return /* @__PURE__ */ jsx13(
    "ellipse",
    {
      cx: 0,
      cy: 0,
      rx: radius[0],
      ry: radius[1],
      strokeWidth: weight,
      strokeDasharray: strokeStyle === "dashed" ? "4,3" : void 0,
      transform: cssTransform,
      ...svgEllipseProps,
      style: {
        stroke: color,
        fill: color,
        fillOpacity,
        strokeOpacity,
        vectorEffect: "non-scaling-stroke",
        ...svgEllipseProps.style || {}
      }
    }
  );
}
Ellipse.displayName = "Ellipse";

// src/display/Circle.tsx
import { jsx as jsx14 } from "react/jsx-runtime";
function Circle({ radius, ...rest }) {
  return /* @__PURE__ */ jsx14(Ellipse, { radius: [radius, radius], ...rest });
}
Circle.displayName = "Circle";

// src/display/Polygon.tsx
import { jsx as jsx15 } from "react/jsx-runtime";
function Polygon({
  points,
  color = Theme.foreground,
  weight = 2,
  fillOpacity = 0.15,
  strokeOpacity = 1,
  strokeStyle = "solid",
  svgPolygonProps = {}
}) {
  const { userTransform } = useTransformContext();
  const scaledPoints = points.map((point) => vec.transform(point, userTransform).join(" ")).join(" ");
  return /* @__PURE__ */ jsx15(
    "polygon",
    {
      points: scaledPoints,
      strokeWidth: weight,
      fillOpacity,
      strokeDasharray: strokeStyle === "dashed" ? "4,3" : void 0,
      strokeLinejoin: "round",
      ...svgPolygonProps,
      style: {
        fill: color,
        fillOpacity,
        stroke: color,
        strokeOpacity,
        vectorEffect: "non-scaling-stroke",
        transform: "var(--mafs-view-transform)",
        ...svgPolygonProps.style || {}
      }
    }
  );
}
Polygon.displayName = "Polygon";

// src/display/Point.tsx
import { jsx as jsx16 } from "react/jsx-runtime";
function Point({
  x,
  y,
  color = Theme.foreground,
  opacity = 1,
  svgCircleProps = {}
}) {
  const { viewTransform: pixelMatrix, userTransform: transform } = useTransformContext();
  const [cx, cy] = vec.transform([x, y], vec.matrixMult(pixelMatrix, transform));
  return /* @__PURE__ */ jsx16(
    "circle",
    {
      cx,
      cy,
      r: 6,
      ...svgCircleProps,
      style: { fill: color, opacity, ...svgCircleProps.style }
    }
  );
}
Point.displayName = "Point";

// src/display/Vector.tsx
import * as React8 from "react";
import { Fragment as Fragment4, jsx as jsx17, jsxs as jsxs4 } from "react/jsx-runtime";
var incrementer2 = 0;
function Vector({
  tail = [0, 0],
  tip,
  color = Theme.foreground,
  weight = 2,
  style = "solid",
  opacity = 1,
  svgLineProps = {}
}) {
  const { userTransform, viewTransform } = useTransformContext();
  const combinedTransform = vec.matrixMult(viewTransform, userTransform);
  const pixelTail = vec.transform(tail, combinedTransform);
  const pixelTip = vec.transform(tip, combinedTransform);
  const id = React8.useMemo(() => `mafs-triangle-${incrementer2++}`, []);
  return /* @__PURE__ */ jsxs4(Fragment4, { children: [
    /* @__PURE__ */ jsx17("defs", { children: /* @__PURE__ */ jsx17("marker", { id, markerWidth: "8", markerHeight: "8", refX: "8", refY: "4", orient: "auto", children: /* @__PURE__ */ jsx17("path", { d: "M 0 0 L 8 4 L 0 8 z", fill: color || "var(--mafs-fg)" }) }) }),
    /* @__PURE__ */ jsx17(
      "line",
      {
        x1: pixelTail[0],
        y1: pixelTail[1],
        x2: pixelTip[0],
        y2: pixelTip[1],
        strokeWidth: weight,
        markerEnd: `url(#${id})`,
        strokeDasharray: style === "dashed" ? "4,3" : void 0,
        ...svgLineProps,
        style: {
          stroke: color || "var(--mafs-fg)",
          strokeOpacity: opacity,
          ...svgLineProps?.style || {},
          vectorEffect: "non-scaling-stroke"
        }
      }
    )
  ] });
}
Vector.displayName = "Vector";

// src/display/Text.tsx
import { jsx as jsx18 } from "react/jsx-runtime";
function Text({
  children,
  x,
  y,
  color,
  size = 30,
  svgTextProps = {},
  attach,
  attachDistance = 0
}) {
  const { viewTransform: pixelMatrix, userTransform: transformContext } = useTransformContext();
  let xOffset = 0;
  let textAnchor = "middle";
  if (attach?.includes("w")) {
    textAnchor = "end";
    xOffset = -1;
  } else if (attach?.includes("e")) {
    textAnchor = "start";
    xOffset = 1;
  }
  let yOffset = 0;
  let dominantBaseline = "middle";
  if (attach?.includes("n")) {
    dominantBaseline = "baseline";
    yOffset = 1;
  } else if (attach?.includes("s")) {
    dominantBaseline = "hanging";
    yOffset = -1;
  }
  let [pixelX, pixelY] = [0, 0];
  if (xOffset !== 0 || yOffset !== 0) {
    ;
    [pixelX, pixelY] = vec.withMag([xOffset, yOffset], attachDistance);
  }
  const center = vec.transform([x, y], vec.matrixMult(pixelMatrix, transformContext));
  return /* @__PURE__ */ jsx18(
    "text",
    {
      x: center[0] + pixelX,
      y: center[1] + pixelY,
      fontSize: size,
      dominantBaseline,
      textAnchor,
      style: {
        fill: color || "var(--mafs-fg)",
        vectorEffect: "non-scaling-stroke"
      },
      className: "mafs-shadow",
      ...svgTextProps,
      children
    }
  );
}
Text.displayName = "Text";

// src/interaction/MovablePoint.tsx
import { useDrag as useDrag2 } from "@use-gesture/react";
import * as React9 from "react";
import invariant4 from "tiny-invariant";
import { jsx as jsx19, jsxs as jsxs5 } from "react/jsx-runtime";
function MovablePoint({
  point,
  onMove,
  constrain = (point2) => point2,
  color = Theme.pink
}) {
  const { viewTransform, userTransform } = useTransformContext();
  const { xSpan, ySpan } = useSpanContext();
  const inverseViewTransform = vec.matrixInvert(viewTransform);
  invariant4(inverseViewTransform, "The view transform must be invertible.");
  const inverseTransform = React9.useMemo(() => getInverseTransform(userTransform), [userTransform]);
  const combinedTransform = React9.useMemo(
    () => vec.matrixMult(viewTransform, userTransform),
    [viewTransform, userTransform]
  );
  const [dragging, setDragging] = React9.useState(false);
  const [displayX, displayY] = vec.transform(point, combinedTransform);
  const pickup = React9.useRef([0, 0]);
  const bind = useDrag2((state) => {
    const { type, event } = state;
    event?.stopPropagation();
    const isKeyboard = type.includes("key");
    if (isKeyboard) {
      event?.preventDefault();
      const { direction: yDownDirection, altKey, metaKey, shiftKey } = state;
      const direction = [yDownDirection[0], -yDownDirection[1]];
      const span = Math.abs(direction[0]) ? xSpan : ySpan;
      let divisions = 50;
      if (altKey || metaKey)
        divisions = 200;
      if (shiftKey)
        divisions = 10;
      const min = span / (divisions * 2);
      const tests = range(span / divisions, span / 2, span / divisions);
      for (const dx of tests) {
        const testMovement = vec.scale(direction, dx);
        const testPoint = constrain(
          vec.transform(
            vec.add(vec.transform(point, userTransform), testMovement),
            inverseTransform
          )
        );
        if (vec.dist(testPoint, point) > min) {
          onMove(testPoint);
          break;
        }
      }
    } else {
      const { last, movement: pixelMovement, first } = state;
      setDragging(!last);
      if (first)
        pickup.current = vec.transform(point, userTransform);
      if (vec.mag(pixelMovement) === 0)
        return;
      const movement = vec.transform(pixelMovement, inverseViewTransform);
      onMove(constrain(vec.transform(vec.add(pickup.current, movement), inverseTransform)));
    }
  });
  const ringSize = 15;
  return /* @__PURE__ */ jsxs5(
    "g",
    {
      ...bind(),
      style: {
        "--movable-point-color": color,
        "--movable-point-ring-size": `${ringSize}px`
      },
      className: `mafs-movable-point ${dragging ? "mafs-movable-point-dragging" : ""}`,
      tabIndex: 0,
      children: [
        /* @__PURE__ */ jsx19("circle", { className: "mafs-movable-point-hitbox", r: 30, cx: displayX, cy: displayY }),
        /* @__PURE__ */ jsx19(
          "circle",
          {
            className: "mafs-movable-point-focus",
            r: ringSize + 1,
            cx: displayX,
            cy: displayY
          }
        ),
        /* @__PURE__ */ jsx19("circle", { className: "mafs-movable-point-ring", r: ringSize, cx: displayX, cy: displayY }),
        /* @__PURE__ */ jsx19("circle", { className: "mafs-movable-point-point", r: 6, cx: displayX, cy: displayY })
      ]
    }
  );
}
MovablePoint.displayName = "MovablePoint";
function getInverseTransform(transform) {
  const invert = vec.matrixInvert(transform);
  invariant4(
    invert !== null,
    "Could not invert transform matrix. Your movable point's transformation matrix might be degenerative (mapping 2D space to a line)."
  );
  return invert;
}

// src/interaction/useMovablePoint.tsx
import * as React10 from "react";
import { jsx as jsx20 } from "react/jsx-runtime";
function useMovablePoint(initialPoint, { constrain, color = Theme.pink } = {}) {
  const [initialX, initialY] = initialPoint;
  const [point, setPoint] = React10.useState(initialPoint);
  const [x, y] = point;
  const constraintFunction = React10.useMemo(() => {
    if (constrain === "horizontal") {
      return ([x2]) => [x2, initialY];
    } else if (constrain === "vertical") {
      return ([, y2]) => [initialX, y2];
    } else if (typeof constrain === "function") {
      return constrain;
    }
    return ([x2, y2]) => [x2, y2];
  }, [constrain, initialX, initialY]);
  const element = React10.useMemo(() => {
    return /* @__PURE__ */ jsx20(MovablePoint, { ...{ point, color }, constrain: constraintFunction, onMove: setPoint });
  }, [point, color, constraintFunction]);
  return {
    x,
    y,
    point: [x, y],
    element,
    setPoint
  };
}

// src/animation/useStopwatch.ts
import * as React11 from "react";
function useStopwatch(options) {
  const { startTime = 0, endTime = Infinity } = options || {};
  const startClockTime = React11.useRef(null);
  const [time, setTime] = React11.useState(startTime);
  const [playing, setPlaying] = React11.useState(false);
  React11.useEffect(() => {
    let request = -1;
    function tick(now) {
      now = now / 1e3;
      if (!startClockTime.current)
        startClockTime.current = now;
      const deltaTime = now - startClockTime.current;
      if (deltaTime >= endTime) {
        startClockTime.current = null;
        setTime(endTime);
        setPlaying(false);
        return;
      }
      setTime(Math.min(deltaTime, endTime));
      request = window.requestAnimationFrame(tick);
    }
    if (playing) {
      request = window.requestAnimationFrame(tick);
    } else {
      window.cancelAnimationFrame(request);
    }
    return () => window.cancelAnimationFrame(request);
  }, [playing, endTime]);
  const start = React11.useCallback(() => setPlaying(true), []);
  const stop = React11.useCallback(() => {
    startClockTime.current = null;
    setPlaying(false);
    setTime(startTime);
  }, [startTime]);
  return { time, setTime: (time2) => setTime(time2 * 1e3), start, stop };
}

// src/display/Transform.tsx
import { jsx as jsx21 } from "react/jsx-runtime";
function Transform(props) {
  const { userTransform, viewTransform } = useTransformContext();
  let builder = vec.matrixBuilder();
  const { matrix, children, ...transforms } = props;
  if (matrix)
    builder = builder.mult(matrix);
  for (const [name, value] of Object.entries(transforms)) {
    if (value == null)
      continue;
    switch (name) {
      case "translate":
        builder = builder.translate(...value);
        break;
      case "scale":
        if (typeof value === "number")
          builder = builder.scale(value, value);
        else
          builder = builder.scale(...value);
        break;
      case "shear":
        builder = builder.shear(...value);
        break;
      case "rotate":
        builder = builder.rotate(value);
        break;
    }
  }
  builder = builder.mult(userTransform);
  const newUserTransform = builder.get();
  return /* @__PURE__ */ jsx21(TransformContext.Provider, { value: { userTransform: newUserTransform, viewTransform }, children: /* @__PURE__ */ jsx21("g", { style: { "--mafs-user-transform": vec.toCSS(newUserTransform) }, children }) });
}
Transform.displayName = "Transform";

// src/debug/ViewportInfo.tsx
import { jsxs as jsxs6 } from "react/jsx-runtime";
function ViewportInfo({ precision = 3 }) {
  const { xMin, xMax, yMin, yMax } = useCoordinateContext();
  const { viewTransform } = useTransformContext();
  const { xPanes, yPanes } = usePaneContext();
  const [x, y] = vec.transform([xMin, yMin], viewTransform);
  const xPanesString = xPanes.map((pane) => `(${pane.join(", ")})`).join("   ");
  const yPanesString = yPanes.map((pane) => `(${pane.join(", ")})`).join("   ");
  return /* @__PURE__ */ jsxs6("g", { className: "mafs-shadow", fontFamily: "monospace", children: [
    /* @__PURE__ */ jsxs6("text", { x: x + 10, y: y - 70, children: [
      "x: (",
      xMin.toFixed(precision),
      ", ",
      xMax.toFixed(precision),
      ")"
    ] }),
    /* @__PURE__ */ jsxs6("text", { x: x + 10, y: y - 50, children: [
      "y: (",
      yMin.toFixed(precision),
      ", ",
      yMax.toFixed(precision),
      ")"
    ] }),
    /* @__PURE__ */ jsxs6("text", { x: x + 10, y: y - 30, children: [
      "xPanes: ",
      xPanesString
    ] }),
    /* @__PURE__ */ jsxs6("text", { x: x + 10, y: y - 10, children: [
      "yPanes: ",
      yPanesString
    ] })
  ] });
}

// src/debug/TransformWidget.tsx
import { Fragment as Fragment5, jsx as jsx22, jsxs as jsxs7 } from "react/jsx-runtime";
function TransformWidget({ children }) {
  const t = useMovablePoint([0, 0]);
  const s = useMovablePoint([1, 1], { color: Theme.blue });
  const r = useMovablePoint([1, 0], {
    color: Theme.green,
    constrain: (p) => vec.normalize(p)
  });
  const angle = Math.atan2(r.point[1], r.point[0]);
  return /* @__PURE__ */ jsxs7(Fragment5, { children: [
    /* @__PURE__ */ jsxs7(Transform, { translate: t.point, children: [
      /* @__PURE__ */ jsxs7(Transform, { rotate: angle, children: [
        /* @__PURE__ */ jsxs7(Transform, { scale: s.point, children: [
          children,
          /* @__PURE__ */ jsx22(
            Polygon,
            {
              points: [
                [0, 0],
                [0, 1],
                [1, 1],
                [1, 0]
              ],
              color: Theme.blue
            }
          )
        ] }),
        /* @__PURE__ */ jsx22(
          Circle,
          {
            center: [0, 0],
            radius: 1,
            strokeStyle: "dashed",
            strokeOpacity: 0.5,
            fillOpacity: 0,
            color: Theme.green
          }
        ),
        s.element
      ] }),
      r.element
    ] }),
    t.element
  ] });
}

// src/debug/index.tsx
var Debug = {
  ViewportInfo,
  TransformWidget
};
Debug.ViewportInfo.displayName = "Debug.ViewportInfo";
Debug.TransformWidget.displayName = "Debug.TransformWidget";
export {
  Circle,
  Coordinates,
  Debug,
  Ellipse,
  Line,
  Mafs,
  MovablePoint,
  Plot,
  Point,
  Polygon,
  Text,
  Theme,
  Transform,
  Vector,
  autoPi as labelPi,
  useMovablePoint,
  usePaneContext,
  useStopwatch,
  useTransformContext,
  vec
};
//# sourceMappingURL=index.mjs.map
